package cn.efunbox.audio.controller;

import cn.efunbox.audio.consts.Status;
import cn.efunbox.audio.entity.Channel;
import cn.efunbox.audio.service.AdminService;
import cn.efunbox.audio.service.ChannelService;
import cn.efunbox.audio.service.TrailService;
import cn.efunbox.audio.utils.ApiCode;
import cn.efunbox.audio.utils.Common;
import cn.efunbox.audio.utils.HttpUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.sql.Timestamp;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by yao on 17-9-26.
 */
@RestController
@Slf4j
@RequestMapping(value = "/channel")
public class ChannelController {

    @Autowired
    ChannelService channelService;
    @Autowired
    AdminService adminService;
    @Autowired
    TrailService trailService;

    @RequestMapping(value = "/search" ,method = RequestMethod.POST)
    public void Search(HttpServletRequest request, HttpServletResponse response){
        String name = request.getParameter("name");
        String idChannel = request.getParameter("idChannel");
        String idFather = request.getParameter("idFather");
        String page = request.getParameter("page");
        String size = request.getParameter("size");
        if(page==null || page.length()<1)
            page = "0";
        if(size==null || size.length()<1)
            size = "0";
        Map<String,Object> map = new HashMap<>();
        List<Channel> list = null;
        if(idChannel!=null && idChannel.length()>0)
            list = channelService.SearchById(Long.valueOf(idChannel));
        else if(name!=null && name.length()>0)
            list = channelService.SearchByName(name);
        else if(idFather!=null && idFather.length()>0)
            list = channelService.SearchByIdFather(Long.valueOf(idFather));
        else{
            Page<Channel> pageRes = channelService.SearchAll(Integer.valueOf(page), Integer.valueOf(size));
            map.put("data", pageRes.getContent());
            map.put("size", pageRes.getNumberOfElements());
            map.put("total", pageRes.getTotalElements());
        }

        if(list!=null){
            map.put("data", list);
            map.put("size", list.size());
            map.put("total", list.size());
        }

        HttpUtil.responseOkData(request, response, map);
        return;
    }

//    @RequestMapping(value = "/insert", method = RequestMethod.POST)
//    public void Insert(HttpServletRequest request, HttpServletResponse response){
//        String name = request.getParameter("name");
//        String idFather = request.getParameter("idFather");
//        if(name==null){
//            HttpUtil.responseApiCode(request, response, ApiCode.PARAMETER_ERROR);
//            return;
//        }
//        if(idFather==null || idFather.length()==0)
//            idFather = "0";
//
//        List<Channel> list = channelService.SearchByName(name);
//        if(list!=null && list.size()>0){
//            HttpUtil.responseApiCode(request, response, ApiCode.RECORD_EXIST);
//            return;
//        }
//        Channel channel = new Channel();
//        channel.setName(name);
//        channel.setIdFather(Long.valueOf(idFather));
//        channel = channelService.Insert(channel);
//        HttpUtil.responseOutWithJson(request, response, channel);
//        return;
//    }

    @RequestMapping(value = "/insert", method = RequestMethod.POST)
    public void Insert(HttpServletRequest request, HttpServletResponse response, @RequestBody Channel channel){
        List<Channel> list = channelService.SearchByName(channel.getName());
        if(list!=null && list.size()>0){
            HttpUtil.responseApiCode(request, response, ApiCode.RECORD_EXIST);
            return;
        }
        channel.setHasSon(false);
        channel.setStatus(Status.ONLINE.getCode());
        channel.setCreated(new Timestamp(new Date().getTime()));
        channel = channelService.Insert(channel);
        if(channel.getIdFather()>0){
            Channel c = channelService.GetOne(channel.getIdFather());
            if(c!=null){
                c.setHasSon(true);
                channelService.Insert(c);
            }
        }

        TrailController.Insert(adminService, trailService, request, "channel", channel.getId(), " 新建渠道：" + channel.getName());

        HttpUtil.responseOkData(request, response, channel);
        return;
    }

    @RequestMapping(value = "/update", method = RequestMethod.POST)
    public void Update(HttpServletRequest request, HttpServletResponse response, @RequestBody Channel channel){
        Channel channelOld = channelService.GetOne(channel.getId());
        if(channelOld==null){
            HttpUtil.responseApiCode(request, response, ApiCode.PARAMETER_ERROR);
            return;
        }
//        System.out.println("ch:"+channel.toString());
//        System.out.println("chOld:"+channelOld.toString());
        channel = (Channel) Common.CopyValue(channel, channelOld);
//        System.out.println("ch2:"+channel.toString());
        if(channelOld.getHasSon())
            channel.setHasSon(true);
        channel = channelService.Insert(channel);
//        System.out.println("ch3:"+channel.toString());

        TrailController.Insert(adminService, trailService, request, "channel", channel.getId(), " 更新渠道：" + channel.getName());

        HttpUtil.responseOkData(request, response, channel);
        return;
    }

    @RequestMapping(value = "/delete", method = RequestMethod.POST)
    public void Delete(HttpServletRequest request, HttpServletResponse response){
        String idChannel = request.getParameter("idChannel");
        Channel channel = channelService.GetOne(Long.valueOf(idChannel));
        if(channel==null){
            HttpUtil.responseApiCode(request, response, ApiCode.PARAMETER_ERROR);
            return;
        }
        channelService.Delete(Long.valueOf(idChannel));
        //判断父节点是否还有子节点
        if(channel.getIdFather()>0){
            List<Channel> channelList = channelService.SearchByIdFather(channel.getIdFather());
            if(channelList==null || channelList.size()==0){
                Channel c = channelService.GetOne(channel.getIdFather());
                if(c!=null){
                    c.setHasSon(false);
                    channelService.Insert(c);
                }
            }
        }

        TrailController.Insert(adminService, trailService, request, "channel", Long.valueOf(idChannel), " 删除渠道：" + channel.getName());

        HttpUtil.responseOk(request, response);
        return;
    }

}
