package cn.efunbox.audio.impl;

import cn.efunbox.audio.entity.Album;
import cn.efunbox.audio.page.OnePage;
import cn.efunbox.audio.repository.AlbumRepo;
import cn.efunbox.audio.service.AlbumService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.math.BigInteger;
import java.util.List;
import java.util.Objects;

/**
 * AlbumServiceImpl
 * Created by xusq on 2018/5/8.
 */
@Service
public class AlbumServiceImpl implements AlbumService {

    @Value("${efunbox.oss.img.url}")
    private String imgURL;

    @Autowired
    private AlbumRepo albumRepo;

    @Override
    public Album GetOne(BigInteger id) {
        return albumRepo.findOne(id);
    }

    @Override
    public OnePage<Album> SearchAll(Album album,Integer page, Integer pageSize) {

        fillSearchAlbum(album);
        long count = albumRepo.count(album);
        OnePage onePage = new OnePage(count,page,pageSize);

        if (count == 0) {
            return onePage;
        }
//        List<Album> albumList = albumRepo.find(album, onePage.getStart(), onePage.getPageSize(), SortHelper.sortMap2Sort(new LinkedHashMap<String, BaseOrderEnum>() {{
//            put("modified", BaseOrderEnum.DESC);
//        }}));
        List<Album> albumList = albumRepo.find(album, onePage.getStart(), onePage.getPageSize(), null);
        albumList.stream().forEach(albumConsumer -> albumConsumer.setImage(imgURL + albumConsumer.getImage()));
        onePage.setList(albumList);
        return onePage;
    }

    private void fillSearchAlbum(Album album) {
        if (Objects.isNull(album.getMediaType())) {
            album.setMediaType(1);
        }
        if (StringUtils.isNotBlank(album.getName())) {
            album.setName("%" + album.getName() + "%");
        }
        if (StringUtils.isNotBlank(album.getAge())) {
            album.setAge("%" + album.getAge() + "%");
        }
        if (StringUtils.isNotBlank(album.getLanguage())) {
            album.setLanguage("%" + album.getLanguage() + "%");
        }
        if (StringUtils.isNotBlank(album.getSubject())) {
            album.setSubject("%" + album.getSubject() + "%");
        }
        if (StringUtils.isNotBlank(album.getSuitableCrowd())) {
            album.setSuitableCrowd("%" + album.getSuitableCrowd() + "%");
        }
        if (StringUtils.isNotBlank(album.getVersion())) {
            album.setVersion("%" + album.getVersion() + "%");
        }
        if (StringUtils.isNotBlank(album.getVolume())) {
            album.setVolume("%" + album.getVolume() + "%");
        }
    }

    @Override
    public Album Insert(Album album) {
        return albumRepo.save(album);
    }

    @Override
    public Album Update(Album album) {
        return albumRepo.update(album);
    }

    @Override
    public List<Album> searchByName(String name) {
        return albumRepo.findByNameLike(name);
    }

    public List<BigInteger> findLikeName(String name) {
        return albumRepo.findIdsByNameLike(name);
    }

}
