package cn.efunbox.audio.impl;

import cn.efunbox.audio.consts.Status;
import cn.efunbox.audio.entity.Album;
import cn.efunbox.audio.entity.Device;
import cn.efunbox.audio.repository.AlbumRepo;
import cn.efunbox.audio.repository.DeviceRepo;
import cn.efunbox.audio.service.DeviceService;
import cn.efunbox.audio.utils.ApiCode;
import cn.efunbox.audio.utils.ApiResult;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.sql.Timestamp;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.UUID;

/**
 * Created by yao on 17-9-26.
 */
@Service
public class DeviceServiceImpl implements DeviceService {

    @Autowired
    DeviceRepo deviceRepo;

    @Autowired
    AlbumRepo albumRepo;

    @Value("${efunbox.oss.img.url}")
    private String imgURL;

    @Override
    public boolean CheckRegister(Long idChannel, String idDevice){
        List<Device> list = deviceRepo.findByChannelDevice(idChannel, idDevice);
        if(list.size()>0)
            return true;
        else
            return false;
    }

    @Override
    public Device Register(Long idChannel, String idDevice){
        if(this.CheckRegister(idChannel, idDevice))
            return null;

        Device device = new Device();
        device.setIdChannel(idChannel);
        device.setIdDevice(idDevice);
        device.setStatus(Status.ONLINE.getCode());
        device.setCreated(new Timestamp(new Date().getTime()));
        device.setUpdated(new Timestamp(new Date().getTime()));
        device.setTokenOld(device.getTokenNew());
        device.setTokenNew(UUID.randomUUID().toString().replaceAll("-", ""));
        device = deviceRepo.save(device);

        return device;
    }

    @Override
    public Device Login(Long id, String token){
        Device device = deviceRepo.findById(id);
        if(device==null)
            return null;
//        System.out.println("token:"+token);
//        System.out.println(device.toString());
        if(device.getTokenNew()!=null && false == device.getTokenNew().equals(token)
            && device.getTokenOld()!=null && false == device.getTokenOld().equals(token))
            return null;
        if(device.getTokenNew().equalsIgnoreCase(token)){
            device.setTokenOld(device.getTokenNew());
            device.setTokenNew(UUID.randomUUID().toString().replaceAll("-", ""));
        }
        device.setUpdated(new Timestamp(new Date().getTime()));
        deviceRepo.save(device);

        return device;
    }

    @Override
    public Device GetOne(Long id){
        Device device = deviceRepo.findById(id);
        return device;
    }

    @Override
    public Page<Device> Search(Long idChannel, String idDevice, int page, int size){
        if(size<=0)
            size = 50;
        Pageable pageable = new PageRequest(page, size);
        Page<Device> list = null;
        if(idChannel>0)
            list = deviceRepo.findByIdChannel(idChannel, pageable);
        else if(StringUtils.isNotBlank(idDevice))
            list = deviceRepo.findByIdDevice(idDevice, pageable);
        else
            list = deviceRepo.findAll(pageable);
        return list;
    }

    @Override
    public Page<Device> SearchAll(int page, int size){
        if(size<=0)
            size = 50;
        Pageable pageable = new PageRequest(page, size);
        Page<Device> list = deviceRepo.findAll(pageable);
        return list;
    }

    @Override
    public Page<Device> Search(Long idChannel, String idDevice, int page, int size, Timestamp start, Timestamp end){
        if(size<=0)
            size = 50;
        Pageable pageable = new PageRequest(page, size);
        Page<Device> list = null;
        if(idChannel>0)
            list = deviceRepo.findByIdChannelAndCreatedBetween(idChannel, start, end, pageable);
        else if(StringUtils.isNotBlank(idDevice))
            list = deviceRepo.findByIdDeviceAndCreatedBetween(idDevice, start, end, pageable);
        else
            list = deviceRepo.findByCreatedBetween(start, end, pageable);
        return list;
    }

    @Override
    public Page<Device> SearchAll(int page, int size, Timestamp start, Timestamp end){
        if(size<=0)
            size = 50;
        Pageable pageable = new PageRequest(page, size);
        Page<Device> list = deviceRepo.findByCreatedBetween(start, end, pageable);
        return list;
    }

    @Override
    public Device Update(Device device){
        device = deviceRepo.save(device);
        return device;
    }

    @Override
    public void Delete(Long id){
        deviceRepo.delete(id);
    }

    @Override
    public Device findToken(Long idChannel, String idDevice) {
        List<Device> devices = deviceRepo.findByChannelDevice(idChannel, idDevice);
        if (!CollectionUtils.isEmpty(devices)) {
            return devices.get(0);
        }
        return null;
    }

    @Override
    public List<Album> findAlbum(String id) {
        Device devices = deviceRepo.findById(Long.valueOf(id));
        if(Objects.isNull(devices)){
            return null;
        }
        List<Album> albums = albumRepo.findAlbum(id);
        albums.stream().forEach(album -> {
            if(Objects.nonNull(album.getImage())){
                album.setImage(imgURL + album.getImage());
            }
        });
        return albumRepo.findAlbum(id);
    }
}
