package cn.efunbox.audio.utils;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Hex;
import org.apache.commons.lang.StringUtils;

import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.*;

/**
 * 常見的几种加密方式
 */
@Slf4j
public class Encrypt {

    public static final String SIGN = "sign";

    public static String createSHA256Sign(Map<String, String> paramMap, String signKey) {

        List<String> sortedKeys = new ArrayList<String>();
        for (Map.Entry<String, String> entry : paramMap.entrySet()) {
            if (SIGN.equals(entry.getKey())) {
                continue;
            }

            sortedKeys.add(entry.getKey());
        }

        if (sortedKeys.size() == 0) {
            // 没有参数
            return "";
        }

        Collections.sort(sortedKeys);

        StringBuffer buff = new StringBuffer("");
        for (String key : sortedKeys) {
            String val = paramMap.get(key);
            if (StringUtils.isBlank(val)) {
                continue;
            }

            buff.append(key).append("=").append(val).append("&");
        }

        buff.append("key=").append(signKey);

        try {
            return Encrypt.String2SHA256(buff.toString());
        } catch (Exception e) {
            throw new RuntimeException("签名错误");
        }
    }

    /**
     *  利用Apache的工具类实现SHA-256加密
     * @param str 加密后的报文
     * @return
     */
    public static String String2SHA256(String str){
        MessageDigest messageDigest;
        String encdeStr = "";
        try {
            messageDigest = MessageDigest.getInstance("SHA-256");
            byte[] hash = messageDigest.digest(str.getBytes("UTF-8"));
            encdeStr = Hex.encodeHexString(hash);
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        return encdeStr;
    }
    //---------
    /**
     * 利用java原生的摘要实现SHA256加密
     * @param str 加密后的报文
     * @return
     */
    public static String String2SHA256StrJava(String str){
        MessageDigest messageDigest;
        String encodeStr = "";
        try {
            messageDigest = MessageDigest.getInstance("SHA-256");
            messageDigest.update(str.getBytes("UTF-8"));
            encodeStr = byte2Hex(messageDigest.digest());
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        return encodeStr;
    }
 
    /**
     * 将byte转为16进制
     * @param bytes
     * @return
     */
    private static String byte2Hex(byte[] bytes){
        StringBuffer stringBuffer = new StringBuffer();
        String temp = null;
        for (int i=0;i<bytes.length;i++){
            temp = Integer.toHexString(bytes[i] & 0xFF);
            if (temp.length()==1){
                //1得到一位的进行补0操作
                stringBuffer.append("0");
            }
            stringBuffer.append(temp);
        }
        return stringBuffer.toString();
    }


    public static void main(String[] args) {
        Map<String,String> param = new HashMap<>();
        param.put("idChannel","1000");
        param.put("idDevice","acf76362e77441fd8329384345d54156");
        String sign = Encrypt.createSHA256Sign(param,"IhOTiTyMLDNNLFuP");
        log.info(sign);
    }
}
